#!/usr/bin/perl -w

# Collect Java performance data when there is a hang, high CPU, slow etc performance issue. 
# 6/13/2008  Wei Luo weiluo@us.ibm.com


if (@ARGV  != 1) {
    print "WARNING Should enter  exactly one arguments\n";
    print "Usage: javaperf.sh <java process id> \n\n";
    exit 1;
}

$javapid=$ARGV[0];

$clct_core = 1;

$javacore = 1;

`ps gewwww | grep $javapid | grep IBM_NOSIGHANDLER=true`;
   if ( $? == 0 ) {
      $javacore = 0;
   }

$psinfo=`ps auw | grep $ARGV[0] | grep -v grep`;

   if ( $? == 0 ) {
      chomp($psinfo);
      print "$psinfo";
      @psinfo=split(/\s+/,$psinfo);
   }

   if ( @psinfo > 1 ) {
      $userid=$psinfo[0];
   }

# Check java process id

 is_javaid(); 
 if ( $? != "0" ) {
   exit 1;
 } 

is_purr();

$workdir=`pwd`;
chomp($workdir);
 
if ( ! -e "$workdir/jperf") {

   system("/usr/bin/mkdir jperf");
   err_check("Can not create directory in the current directory, please check directory permission.\n");
   system("/usr/bin/chmod -R 777 jperf");
   err_check("Can not jperf permission.\n");

} else {

   system("/usr/bin/chmod  777 jperf");
   err_check("Can not change jperf permission.\n");
   system("/usr/bin/rm jperf/*");

}

 
open (LOG, ">> ./jperf/javaperf.log") ;
  if ( $? == 0 ) {
     $openfile = 1;
   }
if ($openfile ) {
   print LOG " Real user id: $userid \n\n";
}

# Start to collect java perf data
 java_perf();
 
# Collect system information
 sys_info();
 
 wrap_up();

#==== Sub routines ====

sub do_timestamp {
    system("/bin/date +%H:%M:%S-%D:");
    print " @_\n\n";

    if ( $openfile ) {
       $t=`/bin/date +%H:%M:%S-%D:`;
       print LOG $t;
       print LOG " @_\n\n";
    }
}

sub is_javaid {
   $_=$ARGV[0];
   $ispid=1;

   if (/[^0-9]/) {
      $ispid=0;
   }
   print "pid $ispid";
   if ( (system("/bin/ps $_ | grep java") ) || !$ispid ) {
      do_timestamp ("The process is $_, this isn\'t a java process id");
      exit 1;
   }  
}

sub err_check {
   if ( $? != "0" ) {
      print "@_ failed.\n";
      if ( $openfile ) {
         print LOG "@_ failed.\n";
      }
   }
}

sub chk_cmds {

    if ( ! -x $_[0] ){
       print("Command: $_[0] can\'t be found\n");
       if ( $openfile ) {
         print LOG "Command: $_[0] can\'t be found\n";
       }
    }
}

sub is_purr {
   $KERTYPE=`bootinfo -K`;
   if ( $KERTYPE == 32 ) {
      $PURR="";
   
   } else {
      if ( system ("/usr/sbin/smtctl >/dev/null")) {
         $PURR="-p";
      } else {
         $PURR="-p -r PURR";
      }
   }
}

sub java_perf {

    do_timestamp("AIX SYSTEM INFORMATION...");

    do_timestamp("Collect vmstat, take 20 seconds...");
    chk_cmds("/usr/bin/vmstat");
    system ("/usr/bin/vmstat 2 10 > ./jperf/vmstat.out");
    err_check("vmstat");    

    do_timestamp("Collect svmon...");
    chk_cmds("/usr/bin/svmon");
    system ("/usr/bin/svmon -P -m -i 2 2 > jperf/svmon.out");
    err_check("svmon");


    do_timestamp("Collect ps avwwg..");
    chk_cmds("/usr/bin/ps");
    system("/usr/bin/ps avwwg> jperf/ps.out");
    err_check("ps avwwg");

    do_timestamp("Collect netstat..");
    chk_cmds("/usr/bin/netstat");
    system("/usr/bin/netstat -an >> netstat.out");
    err_check("netstat -an");

    do_timestamp("Collect tprof...");
    chk_cmds("/usr/bin/tprof");
    system("/usr/bin/tprof -skej -R -x sleep 30");
    err_check("tprof");

    do_timestamp("Collect trace, take 30 seconds...");
    chk_cmds("/usr/bin/trace");
    system("trcstop 2> /dev/null");
    system("/usr/bin/trace -C all -a -n -l $PURR -L100000000 -T50000000 -o jperf/trace.raw");
    err_check("kernel trace"); 

    system("sleep 10");
    err_check("sleep");

    system("trcstop 2> /dev/null");
    err_check("trcstop"); 


    do_timestamp("Collect netstat..");
    chk_cmds("/usr/bin/netstat");
    system("/usr/bin/netstat -Aan >> netstat_Aan.out");
    err_check("netstat -Aan");


    do_timestamp("Collect  AIX core files, may take a while...");
    chk_cmds("/usr/bin/gencore");
    print "user:  $userid\n";
    system("su $userid -c /usr/bin/gencore $javapid core.001");
    err_check("generate AIX core file");

    do_timestamp("Move core.001 to jperf...");
    system("/usr/bin/mv core.001 ./jperf");
    err_check("Move core.001 to jperf");

    system ("sleep 10");

    do_timestamp("Collect pdump...");
    chk_cmds("pdump.sh");
    system("pdump.sh $javapid ");
    err_check("Collect pdump");


    if ( $javacore ) {
       do_timestamp("Collect first javacore files..");
       system("kill -3 $javapid");
       err_check("generate first javacore file");
    }

     system ("sleep 10"); 

    if ( $javacore ) {
       do_timestamp("Collect second javacore file..");
       system("kill -3 $javapid");
       err_check("generate second javacore file");
    }

    system ("sleep 30");

    if ( $clct_core ) {
       do_timestamp("Collect second AIX core files, may take a while...");
       chk_cmds("/usr/bin/gencore");
       system("su $userid -c /usr/bin/gencore $javapid jperf/core.002");
       err_check("generate second core file");
       system("/usr/bin/mv core.002 ./jperf");
    }

    do_timestamp("Collect second netstat..");
    chk_cmds("/usr/bin/netstat");
    system("/usr/bin/netstat -an >> netstat.out");
    err_check("netstat -an");

    system ("sleep 10");

    if ( $javacore ) {
       do_timestamp("Collect third javacore files..");
       system("kill -3 $javapid");
       err_check("generate third javacore file");
    }


    do_timestamp("Collect pdump...");
    chk_cmds("pdump.sh");
    system("pdump.sh $javapid ");
    err_check("Collect pdump");

    system("/usr/bin/mv pdump.java.$javapid.*.out ./jperf");
}


sub sys_info {


   do_timestamp("Collect trcnm...");
   system("/usr/bin/trcnm  > ./jperf/trcnm.out");
   err_check("Collect trcnm");                                                      

   do_timestamp("Collect gennames...");
   system("/usr/bin/gennames > jperf/gennames.out");
   err_check("Collect gennames");

   system("/usr/bin/gensyms > jperf/gensyms.out");
   err_check("Collect gensyms");

   system("/usr/bin/cp /etc/trcfmt jperf/trcfmt.out");
   err_check("Collect trcfmt");

   do_timestamp("Collect errpt...");
   system("/usr/bin/errpt -a > jperf/errpt.out");
   err_check("Collect errpt");

   do_timestamp("Collect lslpp...");
   system("/usr/bin/lslpp -lc > jperf/lslpp.out");
   err_check("Collect lslpp");

   do_timestamp("Collect instfix...");
   system("/usr/sbin/instfix -i > jperf/instfix.out");
   err_check("Collect instfix");

   do_timestamp("Collect bootinfo...");
   system("/usr/sbin/bootinfo -K > jperf/bootinfo.out");
   err_check("Collect instfix");

   do_timestamp("Collect lsattr...");
   system("/usr/sbin/lsattr -El sys0 > jperf/lsattr.out");
   err_check("Collect lsattr");

   do_timestamp("Collect lsps...");
   system("/usr/sbin/lsps -s > jperf/lsps.out");
   err_check("Collect lsps");

}

sub wrap_up {
   do_timestamp("run libGrabber...");
   system("./javaLibsGrabber.sh jperf/core.001");
   err_check("Collect libs for core.001");

   #do_timestamp("run jextract ....");
   #system("/opt/compdev/comp/vendor/JRE16_32_SR7/jre/bin/jextract jperf/core.001");
   #err_check("Collect jextract  for core.001");

   if ( $clct_core ) {
      system("./javaLibsGrabber.sh jperf/core.002");
      err_check("Collect libs for core.002");
   }
  
   close (LOG) || die "can't close javaperf.log";
}
